/* JPVMBuffer.java
 *
 * Classes to implement a packable / unpackable buffer that can
 * be used to store the payload associated with a message.
 *
 * Adam J Ferrari
 * Sun 05-26-1996
 *
 * Copyright (C) 1996  Adam J Ferrari
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 * MA 02139, USA.
 */

package jpvm;

import jpvm.JPVMException;
import jpvm.JPVMDataType;
import jpvm.JPVMDebug;
import jpvm.JPVMSendConnection;
import jpvm.JPVMRecvConnection;
import jpvm.JPVMTaskId;
import java.io.*;

public class JPVMBuffer
{
	private JPVMBufferElement	list_head;
	private JPVMBufferElement	list_tail;
	private JPVMBufferElement	curr_elt;
	private int					num_list_elts;

	private void addElt(JPVMBufferElement nw)
	{
		num_list_elts++;
		if (list_head == null)
		{
			curr_elt = list_head = list_tail = nw;
			return;
		}
		list_tail.next = nw;
		list_tail = nw;
	}

	public JPVMBuffer()
	{
		list_head = null;
		list_tail = null;
		num_list_elts = 0;
	}

	public void rewind()
	{
		curr_elt = list_head;
	}

	public void pack(int d[], int n, int stride)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d, n, stride);
		addElt(nw);
	}

	public void pack(int d)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d);
		addElt(nw);
	}

	public void pack(char d[], int n, int stride)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d, n, stride);
		addElt(nw);
	}

	public void pack(char d)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d);
		addElt(nw);
	}

	public void pack(short d[], int n, int stride)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d, n, stride);
		addElt(nw);
	}

	public void pack(short d)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d);
		addElt(nw);
	}

	public void pack(long d[], int n, int stride)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d, n, stride);
		addElt(nw);
	}

	public void pack(long d)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d);
		addElt(nw);
	}

	public void pack(byte d[], int n, int stride)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d, n, stride);
		addElt(nw);
	}

	public void pack(byte d)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d);
		addElt(nw);
	}

	public void pack(float d[], int n, int stride)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d, n, stride);
		addElt(nw);
	}

	public void pack(float d)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d);
		addElt(nw);
	}

	public void pack(double d[], int n, int stride)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d, n, stride);
		addElt(nw);
	}

	public void pack(double d)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d);
		addElt(nw);
	}

	public void pack(JPVMTaskId d[], int n, int stride)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d, n, stride);
		addElt(nw);
	}

	public void pack(JPVMTaskId d)
	{
		JPVMBufferElement nw = new JPVMBufferElement(d);
		addElt(nw);
	}

	public void pack(String str)
	{
		JPVMBufferElement nw = new JPVMBufferElement(str);
		addElt(nw);
	}

	public void unpack(int d[], int n, int stride) throws JPVMException
	{
		if (curr_elt == null)
			throw new JPVMException("buffer empty, upkint.");
		curr_elt.unpack(d, n, stride);
		curr_elt = curr_elt.next;
	}

	public int upkint() throws JPVMException
	{
		int d[] = new int[1];
		unpack(d, 1, 1);
		return d[0];
	}

	public void unpack(byte d[], int n, int stride) throws JPVMException
	{
		if (curr_elt == null)
			throw new JPVMException("buffer empty, upkbyte.");
		curr_elt.unpack(d, n, stride);
		curr_elt = curr_elt.next;
	}

	public byte upkbyte() throws JPVMException
	{
		byte d[] = new byte[1];
		unpack(d, 1, 1);
		return d[0];
	}

	public void unpack(char d[], int n, int stride) throws JPVMException
	{
		if (curr_elt == null)
			throw new JPVMException("buffer empty, upkchar.");
		curr_elt.unpack(d, n, stride);
		curr_elt = curr_elt.next;
	}

	public char upkchar() throws JPVMException
	{
		char d[] = new char[1];
		unpack(d, 1, 1);
		return d[0];
	}

	public void unpack(short d[], int n, int stride) throws JPVMException
	{
		if (curr_elt == null)
			throw new JPVMException("buffer empty, upkshort.");
		curr_elt.unpack(d, n, stride);
		curr_elt = curr_elt.next;
	}

	public short upkshort() throws JPVMException
	{
		short d[] = new short[1];
		unpack(d, 1, 1);
		return d[0];
	}

	public void unpack(long d[], int n, int stride) throws JPVMException
	{
		if (curr_elt == null)
			throw new JPVMException("buffer empty, upklong.");
		curr_elt.unpack(d, n, stride);
		curr_elt = curr_elt.next;
	}

	public long upklong() throws JPVMException
	{
		long d[] = new long[1];
		unpack(d, 1, 1);
		return d[0];
	}

	public void unpack(float d[], int n, int stride) throws JPVMException
	{
		if (curr_elt == null)
			throw new JPVMException("buffer empty, upkfloat.");
		curr_elt.unpack(d, n, stride);
		curr_elt = curr_elt.next;
	}

	public float upkfloat() throws JPVMException
	{
		float d[] = new float[1];
		unpack(d, 1, 1);
		return d[0];
	}

	public void unpack(double d[], int n, int stride) throws JPVMException
	{
		if (curr_elt == null)
			throw new JPVMException("buffer empty, upkdouble.");
		curr_elt.unpack(d, n, stride);
		curr_elt = curr_elt.next;
	}

	public double upkdouble() throws JPVMException
	{
		double d[] = new double[1];
		unpack(d, 1, 1);
		return d[0];
	}

	public void unpack(JPVMTaskId d[], int n, int stride) throws JPVMException
	{
		if (curr_elt == null)
			throw new JPVMException("buffer empty, upktid.");
		curr_elt.unpack(d, n, stride);
		curr_elt = curr_elt.next;
	}

	public JPVMTaskId upktid() throws JPVMException
	{
		JPVMTaskId d[] = new JPVMTaskId[1];
		unpack(d, 1, 1);
		return d[0];
	}

	public String upkstr() throws JPVMException
	{
		if (curr_elt == null)
			throw new JPVMException("buffer empty, upkstring.");
		String ret = curr_elt.unpack();
		curr_elt = curr_elt.next;
		return ret;
	}

	public void send(JPVMSendConnection conn) throws JPVMException
	{
		DataOutputStream strm = conn.strm;
		JPVMBufferElement tmp = list_head;
		try
		{
			strm.writeInt(num_list_elts);
			while (tmp != null)
			{
				tmp.send(conn);
				tmp = tmp.next;
			}
		}
		catch (IOException ioe)
		{
			JPVMDebug.note("JPVMBuffer, send - i/o exception");
			throw new JPVMException("JPVMBuffer, send - " + "i/o exception");
		}
	}

	public void recv(JPVMRecvConnection conn) throws JPVMException
	{
		int i, N;
		JPVMBufferElement tmp;

		DataInputStream strm = conn.strm;
		try
		{
			N = strm.readInt();
			JPVMDebug.note("JPVMBuffer, recv " + N + " buffer elements.");
			for (i = 0; i < N; i++)
			{
				tmp = new JPVMBufferElement();
				tmp.recv(conn);
				addElt(tmp);
			}
		}
		catch (IOException ioe)
		{
			JPVMDebug.note("JPVMBuffer, recv - i/o exception");
			throw new JPVMException("JPVMBuffer, recv - " + "i/o exception");
		}
	}
};

class JPVMBufferElementContents implements Serializable
{
	public int			dataType;
	public int			arraySize;
	public byte			byteArray[];
	public char			charArray[];
	public int			intArray[];
	public short		shortArray[];
	public long			longArray[];
	public float		floatArray[];
	public double		doubleArray[];
	public JPVMTaskId	taskArray[];

	private void init()
	{
		dataType = JPVMDataType.jpvmNull;
		arraySize = 0;
		byteArray = null;
		charArray = null;
		shortArray = null;
		intArray = null;
		longArray = null;
		floatArray = null;
		doubleArray = null;
		taskArray = null;
	}

	public JPVMBufferElementContents(JPVMTaskId d[], int n, int stride,	boolean inPlace)
	{
		init();
		dataType = JPVMDataType.jpvmTid;
		if (stride == 1)
		{
			if (inPlace)
				taskArray = d;
			else
			{
				taskArray = new JPVMTaskId[n];
				System.arraycopy(d, 0, taskArray, 0, n);
			}
		}
		else
		{
			taskArray = new JPVMTaskId[n];
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				taskArray[i] = d[j];
		}
	}

	public JPVMBufferElementContents(short d[], int n, int stride,
			boolean inPlace)
	{
		init();
		dataType = JPVMDataType.jpvmShort;
		if (stride == 1)
		{
			if (inPlace)
				shortArray = d;
			else
			{
				shortArray = new short[n];
				System.arraycopy(d, 0, shortArray, 0, n);
			}
		}
		else
		{
			shortArray = new short[n];
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				shortArray[i] = d[j];
		}
	}

	public JPVMBufferElementContents(int d[], int n, int stride, boolean inPlace)
	{
		init();
		dataType = JPVMDataType.jpvmInteger;
		if (stride == 1)
		{
			if (inPlace)
				intArray = d;
			else
			{
				intArray = new int[n];
				System.arraycopy(d, 0, intArray, 0, n);
			}
		}
		else
		{
			intArray = new int[n];
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				intArray[i] = d[j];
		}
	}

	public JPVMBufferElementContents(long d[], int n, int stride,
			boolean inPlace)
	{
		init();
		dataType = JPVMDataType.jpvmLong;
		if (stride == 1)
		{
			if (inPlace)
				longArray = d;
			else
			{
				longArray = new long[n];
				System.arraycopy(d, 0, longArray, 0, n);
			}
		}
		else
		{
			longArray = new long[n];
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				longArray[i] = d[j];
		}
	}

	public JPVMBufferElementContents(char d[], int n, int stride,
			boolean inPlace)
	{
		init();
		dataType = JPVMDataType.jpvmChar;
		if (stride == 1)
		{
			if (inPlace)
				charArray = d;
			else
			{
				charArray = new char[n];
				System.arraycopy(d, 0, charArray, 0, n);
			}
		}
		else
		{
			charArray = new char[n];
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				charArray[i] = d[j];
		}
	}

	public JPVMBufferElementContents(float d[], int n, int stride,
			boolean inPlace)
	{
		init();
		dataType = JPVMDataType.jpvmFloat;
		if (stride == 1)
		{
			if (inPlace)
				floatArray = d;
			else
			{
				floatArray = new float[n];
				System.arraycopy(d, 0, floatArray, 0, n);
			}
		}
		else
		{
			floatArray = new float[n];
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				floatArray[i] = d[j];
		}
	}

	public JPVMBufferElementContents(double d[], int n, int stride,
			boolean inPlace)
	{
		init();
		dataType = JPVMDataType.jpvmDouble;
		if (stride == 1)
		{
			if (inPlace)
				doubleArray = d;
			else
			{
				doubleArray = new double[n];
				System.arraycopy(d, 0, doubleArray, 0, n);
			}
		}
		else
		{
			doubleArray = new double[n];
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				doubleArray[i] = d[j];
		}
	}

	public JPVMBufferElementContents(byte d[], int n, int stride,
			boolean inPlace)
	{
		init();
		dataType = JPVMDataType.jpvmByte;
		if (stride == 1)
		{
			if (inPlace)
				byteArray = d;
			else
			{
				byteArray = new byte[n];
				System.arraycopy(d, 0, byteArray, 0, n);
			}
		}
		else
		{
			byteArray = new byte[n];
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				byteArray[i] = d[j];
		}
	}

	public void unpack(int d[], int n, int stride) throws JPVMException
	{
		if (dataType != JPVMDataType.jpvmInteger)
		{
			throw new JPVMException("buffer type mismatch, upkint.");
		}
		if (stride == 1)
		{
			System.arraycopy(intArray, 0, d, 0, n);
		}
		else
		{
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				d[j] = intArray[i];
		}
	}

	public void unpack(short d[], int n, int stride) throws JPVMException
	{
		if (dataType != JPVMDataType.jpvmShort)
		{
			throw new JPVMException("buffer type mismatch, upkshort.");
		}
		if (stride == 1)
		{
			System.arraycopy(shortArray, 0, d, 0, n);
		}
		else
		{
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				d[j] = shortArray[i];
		}
	}

	public void unpack(byte d[], int n, int stride) throws JPVMException
	{
		if (dataType != JPVMDataType.jpvmByte)
		{
			throw new JPVMException("buffer type mismatch, upkbyte.");
		}
		if (stride == 1)
			System.arraycopy(byteArray, 0, d, 0, n);
		else
		{
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				d[j] = byteArray[i];
		}
	}

	public void unpack(char d[], int n, int stride) throws JPVMException
	{
		if (dataType != JPVMDataType.jpvmChar)
		{
			throw new JPVMException("buffer type mismatch, upkchar.");
		}
		if (stride == 1)
		{
			System.arraycopy(charArray, 0, d, 0, n);
		}
		else
		{
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				d[j] = charArray[i];
		}
	}

	public void unpack(long d[], int n, int stride) throws JPVMException
	{
		if (dataType != JPVMDataType.jpvmLong)
		{
			throw new JPVMException("buffer type mismatch, upklong.");
		}
		if (stride == 1)
		{
			System.arraycopy(longArray, 0, d, 0, n);
		}
		else
		{
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				d[j] = longArray[i];
		}
	}

	public void unpack(double d[], int n, int stride) throws JPVMException
	{
		if (dataType != JPVMDataType.jpvmDouble)
		{
			throw new JPVMException("buffer type mismatch, upkdouble.");
		}
		if (stride == 1)
		{
			System.arraycopy(doubleArray, 0, d, 0, n);
		}
		else
		{
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				d[j] = doubleArray[i];
		}
	}

	public void unpack(float d[], int n, int stride) throws JPVMException
	{
		if (dataType != JPVMDataType.jpvmFloat)
		{
			throw new JPVMException("buffer type mismatch, upkfloat.");
		}
		if (stride == 1)
		{
			System.arraycopy(floatArray, 0, d, 0, n);
		}
		else
		{
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				d[j] = floatArray[i];
		}
	}

	public void unpack(JPVMTaskId d[], int n, int stride) throws JPVMException
	{
		if (dataType != JPVMDataType.jpvmTid)
		{
			throw new JPVMException("buffer type mismatch, upktid.");
		}
		if (stride == 1)
		{
			System.arraycopy(taskArray, 0, d, 0, n);
		}
		else
		{
			int i, j;
			for (i = 0, j = 0; i < n; i++, j += stride)
				d[j] = taskArray[i];
		}
	}

	public String unpack() throws JPVMException
	{
		if (dataType != JPVMDataType.jpvmString)
		{
			throw new JPVMException("buffer type mismatch, upkstring.");
		}
		return new String(charArray);
	}
};

class JPVMBufferElement
{
	private boolean						inPlace;
	public JPVMBufferElementContents	contents;
	public JPVMBufferElement			next;		// Linked structure

	public void init()
	{
		contents = null;
		next = null;
	}

	public JPVMBufferElement()
	{
		init();
		inPlace = false;
	}

	public JPVMBufferElement(boolean dataInPlace)
	{
		init();
		inPlace = dataInPlace;
	}

	public JPVMBufferElement(JPVMTaskId d[], int n, int stride)
	{
		init();
		contents = new JPVMBufferElementContents(d, n, stride, inPlace);
	}

	public JPVMBufferElement(JPVMTaskId d)
	{
		init();
		JPVMTaskId a[] = new JPVMTaskId[1];
		a[0] = d;
		contents = new JPVMBufferElementContents(a, 1, 1, true);
	}

	public JPVMBufferElement(byte d[], int n, int stride)
	{
		init();
		contents = new JPVMBufferElementContents(d, n, stride, inPlace);
	}

	public JPVMBufferElement(byte d)
	{
		init();
		byte a[] = new byte[1];
		a[0] = d;
		contents = new JPVMBufferElementContents(a, 1, 1, true);
	}

	public JPVMBufferElement(short d[], int n, int stride)
	{
		init();
		contents = new JPVMBufferElementContents(d, n, stride, inPlace);
	}

	public JPVMBufferElement(short d)
	{
		init();
		short a[] = new short[1];
		a[0] = d;
		contents = new JPVMBufferElementContents(a, 1, 1, true);
	}

	public JPVMBufferElement(char d[], int n, int stride)
	{
		init();
		contents = new JPVMBufferElementContents(d, n, stride, inPlace);
	}

	public JPVMBufferElement(char d)
	{
		init();
		char a[] = new char[1];
		a[0] = d;
		contents = new JPVMBufferElementContents(a, 1, 1, true);
	}

	public JPVMBufferElement(long d[], int n, int stride)
	{
		init();
		contents = new JPVMBufferElementContents(d, n, stride, inPlace);
	}

	public JPVMBufferElement(long d)
	{
		init();
		long a[] = new long[1];
		a[0] = d;
		contents = new JPVMBufferElementContents(a, 1, 1, true);
	}

	public JPVMBufferElement(int d[], int n, int stride)
	{
		init();
		contents = new JPVMBufferElementContents(d, n, stride, inPlace);
	}

	public JPVMBufferElement(int d)
	{
		init();
		int a[] = new int[1];
		a[0] = d;
		contents = new JPVMBufferElementContents(a, 1, 1, true);
	}

	public JPVMBufferElement(float d[], int n, int stride)
	{
		init();
		contents = new JPVMBufferElementContents(d, n, stride, inPlace);
	}

	public JPVMBufferElement(float d)
	{
		init();
		float a[] = new float[1];
		a[0] = d;
		contents = new JPVMBufferElementContents(a, 1, 1, true);
	}

	public JPVMBufferElement(double d[], int n, int stride)
	{
		init();
		contents = new JPVMBufferElementContents(d, n, stride, inPlace);
	}

	public JPVMBufferElement(double d)
	{
		init();
		double a[] = new double[1];
		a[0] = d;
		contents = new JPVMBufferElementContents(a, 1, 1, true);
	}

	public JPVMBufferElement(String str)
	{
		init();
		int n = str.length();
		char a[] = new char[n];
		str.getChars(0, n, a, 0);
		contents = new JPVMBufferElementContents(a, n, 1, true);
		contents.dataType = JPVMDataType.jpvmString;
	}

	public void unpack(int d[], int n, int stride) throws JPVMException
	{
		contents.unpack(d, n, stride);
	}

	public void unpack(short d[], int n, int stride) throws JPVMException
	{
		contents.unpack(d, n, stride);
	}

	public void unpack(byte d[], int n, int stride) throws JPVMException
	{
		contents.unpack(d, n, stride);
	}

	public void unpack(char d[], int n, int stride) throws JPVMException
	{
		contents.unpack(d, n, stride);
	}

	public void unpack(long d[], int n, int stride) throws JPVMException
	{
		contents.unpack(d, n, stride);
	}

	public void unpack(double d[], int n, int stride) throws JPVMException
	{
		contents.unpack(d, n, stride);
	}

	public void unpack(float d[], int n, int stride) throws JPVMException
	{
		contents.unpack(d, n, stride);
	}

	public void unpack(JPVMTaskId d[], int n, int stride) throws JPVMException
	{
		contents.unpack(d, n, stride);
	}

	public String unpack() throws JPVMException
	{
		return contents.unpack();
	}

	public void send(JPVMSendConnection conn) throws JPVMException
	{
		int i;
		try
		{
			ObjectOutputStream out;
			out = new ObjectOutputStream(conn.strm);
			out.writeObject(contents);
			// out.flush();
		}
		catch (IOException ioe)
		{
			System.err.println("I/O exception - " + ioe);
			JPVMDebug.note("JPVMBufferElement, " + "send - i/o exception");
			throw new JPVMException("JPVMBufferElement, "
					+ "send - i/o exception");
		}
	}

	public void recv(JPVMRecvConnection conn) throws JPVMException
	{
		int i;
		try
		{
			ObjectInputStream in;
			in = new ObjectInputStream(conn.strm);
			try
			{
				contents = (JPVMBufferElementContents) in.readObject();
			}
			catch (ClassNotFoundException cnf)
			{
				throw new JPVMException("JPVMBufferElement, "
						+ "recv - can't find class "
						+ "JPVMBufferElementContents");
			}
		}
		catch (IOException ioe)
		{
			JPVMDebug.note("JPVMBufferElement, " + "recv - i/o exception");
			throw new JPVMException("JPVMBufferElement, "
					+ "recv - i/o exception");
		}
	}
};
