/* JPVMConnectionSet.java
 *
 * A class to manage the storage of a set of send and receive
 * connections to other jpvm tasks.
 *
 * Adam J Ferrari
 * Sun 05-26-1996
 *
 * Copyright (C) 1996  Adam J Ferrari
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 * MA 02139, USA.
 */

package jpvm;

import jpvm.JPVMSendConnection;
import jpvm.JPVMRecvConnection;
import jpvm.JPVMTaskId;

class jpvmSendConnectionListNode
{
	public JPVMSendConnection			conn;
	public jpvmSendConnectionListNode	next;

	public jpvmSendConnectionListNode()
	{
		conn = null;
		next = null;
	}

	public jpvmSendConnectionListNode(JPVMSendConnection c)
	{
		conn = c;
		next = null;
	}
};

class JPVMRecvConnectionListNode
{
	public JPVMRecvConnection			conn;
	public JPVMRecvConnectionListNode	next;

	public JPVMRecvConnectionListNode()
	{
		conn = null;
		next = null;
	}

	public JPVMRecvConnectionListNode(JPVMRecvConnection c)
	{
		conn = c;
		next = null;
	}
};

public class JPVMConnectionSet
{
	private JPVMRecvConnectionListNode	recvList;
	private jpvmSendConnectionListNode	sendList;
	private JPVMRecvConnectionListNode	recvListIter;

	JPVMConnectionSet()
	{
		recvList = null;
		sendList = null;
	}

	public synchronized JPVMRecvConnection lookupRecvConnection(JPVMTaskId tid)
	{
		JPVMRecvConnectionListNode tmp = recvList;
		while (tmp != null)
		{
			if (tmp.conn.tid.equals(tid))
				return tmp.conn;
			tmp = tmp.next;
		}
		return null;
	}

	public synchronized void insertRecvConnection(JPVMRecvConnection c)
	{
		JPVMRecvConnectionListNode nw;
		nw = new JPVMRecvConnectionListNode(c);
		nw.next = recvList;
		recvList = nw;
	}

	public synchronized JPVMRecvConnection firstIterRecv()
	{
		recvListIter = recvList;
		if (recvListIter != null)
			return recvListIter.conn;
		return null;
	}

	public synchronized JPVMRecvConnection nextIterRecv()
	{
		if (recvListIter != null)
			recvListIter = recvListIter.next;
		if (recvListIter != null)
			return recvListIter.conn;
		return null;
	}

	public synchronized JPVMSendConnection lookupSendConnection(JPVMTaskId tid)
	{
		jpvmSendConnectionListNode tmp = sendList;
		while (tmp != null)
		{
			if (tmp.conn.tid.equals(tid))
				return tmp.conn;
			tmp = tmp.next;
		}
		return null;
	}

	public synchronized void insertSendConnection(JPVMSendConnection c)
	{
		jpvmSendConnectionListNode nw;
		nw = new jpvmSendConnectionListNode(c);
		nw.next = sendList;
		sendList = nw;
	}

};
