/* JPVMMessage
 *
 * A class representing a message in the jpvm system. Messages
 * can be either sent or received to or from other jpvm tasks.
 *
 * Adam J Ferrari
 * Sun 05-26-1996
 *
 * Copyright (C) 1996  Adam J Ferrari
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 * MA 02139, USA.
 */

package jpvm;

import jpvm.JPVMDataType;
import jpvm.JPVMException;
import jpvm.JPVMBuffer;
import jpvm.JPVMTaskId;
import java.io.*;

public class JPVMMessage
{
	private int			messageTag;
	private JPVMTaskId	sourceTid;
	private JPVMTaskId	destTid;
	private JPVMBuffer	buffer;

	public JPVMMessage()
	{
		messageTag = -1;
		sourceTid = null;
		destTid = null;
		buffer = null;
	}

	public JPVMMessage(JPVMBuffer buf, JPVMTaskId dest, JPVMTaskId src, int tag)
	{
		messageTag = tag;
		sourceTid = src;
		destTid = dest;
		buffer = buf;
	}

	public JPVMMessage(JPVMRecvConnection conn) throws JPVMException
	{
		messageTag = -1;
		sourceTid = null;
		destTid = null;
		buffer = null;
		recv(conn);
	}

	public void send(JPVMSendConnection conn) throws JPVMException
	{
		DataOutputStream strm = conn.strm;
		try
		{
			strm.writeInt(messageTag);
			sourceTid.send(strm);
			destTid.send(strm);
			buffer.send(conn);
			strm.flush();
		}
		catch (IOException ioe)
		{
			JPVMDebug.note("JPVMMessage, send - i/o exception");
			throw new JPVMException("JPVMMessage, send - " + "i/o exception");
		}
	}

	public void recv(JPVMRecvConnection conn) throws JPVMException
	{
		DataInputStream strm = conn.strm;
		try
		{
			messageTag = strm.readInt();
			sourceTid = new JPVMTaskId();
			sourceTid.recv(strm);
			destTid = new JPVMTaskId();
			destTid.recv(strm);
			buffer = new JPVMBuffer();
			buffer.recv(conn);
		}
		catch (IOException ioe)
		{
			JPVMDebug.note("JPVMMessage, recv - i/o exception");
			throw new JPVMException("JPVMMessage, recv - " + "i/o exception");
		}
	}

	public JPVMBuffer getBuffer()
	{
		return buffer;
	}

	public void setBuffer(JPVMBuffer buffer)
	{
		this.buffer = buffer;
	}

	public JPVMTaskId getDestTid()
	{
		return destTid;
	}

	public void setDestTid(JPVMTaskId destTid)
	{
		this.destTid = destTid;
	}

	public int getMessageTag()
	{
		return messageTag;
	}

	public void setMessageTag(int messageTag)
	{
		this.messageTag = messageTag;
	}

	public JPVMTaskId getSourceTid()
	{
		return sourceTid;
	}

	public void setSourceTid(JPVMTaskId sourceTid)
	{
		this.sourceTid = sourceTid;
	}
	
	
};
