/* JPVMRecvConnection.java
 *
 * The JPVMRecvConnection class implements objects that represent
 * connections to remote jpvm processes from which messages may be
 * received.
 *
 * Adam J Ferrari
 * Sun 05-26-1996
 *
 * Copyright (C) 1996  Adam J Ferrari
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 * MA 02139, USA.
 */

package jpvm;

import java.net.*;
import java.io.*;
import jpvm.JPVMTaskId;

public class JPVMRecvConnection
{
	private InputStream		instrm;
	public DataInputStream	strm;
	public JPVMTaskId		tid;

	public JPVMRecvConnection()
	{
		instrm = null;
		strm = null;
		tid = null;
	}

	public JPVMRecvConnection(Socket sock)
	{
		if (sock == null)
			return;
		try
		{
			instrm = sock.getInputStream();
			instrm = new BufferedInputStream(instrm);
			strm = new DataInputStream(instrm);
			tid = new JPVMTaskId();
			try
			{
				tid.recv(strm);
			}
			catch (JPVMException jpe)
			{
				strm = null;
				tid = null;
				JPVMDebug.error("JPVMRecvConnection, internal" + " error");
			}
			JPVMDebug.note("JPVMRecvConnection, connect to " + tid.toString()
					+ " established");
		}
		catch (IOException ioe)
		{
			strm = null;
			tid = null;
			JPVMDebug.error("JPVMRecvConnection, i/o exception");
		}
		if (strm == null)
			return;
	}

	public boolean hasMessagesQueued()
	{
		boolean ret = false;
		if (instrm != null)
		{
			try
			{
				if (instrm.available() > 0)
					ret = true;
			}
			catch (IOException ioe)
			{
				ret = false;
				JPVMDebug.error("JPVMRecvConnection, "
						+ "hasMessagesQueued - i/o exception");
			}
		}
		return ret;
	}
};
