/* JPVMRecvThread.java
 *
 * A JPVMRecvThread is a thread of control in a JPVM program
 * responsible for receiving messages on a single receive
 * connection and enqueing them on a massage queue.
 *
 * Adam J Ferrari
 * Tue 06-04-1996
 *
 * Copyright (C) 1996  Adam J Ferrari
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 * MA 02139, USA.
 */

package jpvm;

import jpvm.JPVMException;
import jpvm.JPVMMessage;
import jpvm.JPVMRecvConnection;
import jpvm.JPVMMessageQueue;
import jpvm.JPVMDebug;

public class JPVMRecvThread extends Thread
{
	private JPVMRecvConnection	conn;
	private JPVMMessageQueue	queue;
	int							my_num;
	static int					num	= 0;

	public JPVMRecvThread(JPVMRecvConnection c, JPVMMessageQueue q)
	{
		conn = c;
		queue = q;
		num++;
		my_num = num;
	}

	public void run()
	{
		boolean alive = true;
		while (alive)
		{
			try
			{
				JPVMDebug.note("JPVMRecvThread (" + my_num + ") - blocking "
						+ "for a message.");
				JPVMMessage nw = new JPVMMessage(conn);
				JPVMDebug.note("JPVMRecvThread (" + my_num + ") - got a "
						+ "new message.");
				queue.enqueue(nw);
				Thread.yield();
			}
			catch (JPVMException jpe)
			{
				JPVMDebug.note("JPVMRecvThread, " + "connection closed");
				alive = false;
			}
		}
	}
};
