/* JPVMSendConnection.java
 *
 * The JPVMSendConnection class implements objects that represent
 * connections to remote jpvm processes to which messages may be
 * sent.
 *
 * Adam J Ferrari
 * Sun 05-26-1996
 *
 * Copyright (C) 1996  Adam J Ferrari
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 * MA 02139, USA.
 */

package jpvm;

import java.net.*;
import java.io.*;
import jpvm.JPVMTaskId;

public class JPVMSendConnection
{
	public DataOutputStream	strm;
	public JPVMTaskId		tid;

	public JPVMSendConnection()
	{
		strm = null;
		tid = null;
	}

	public JPVMSendConnection(Socket sock, JPVMTaskId t)
	{
		if (sock == null || t == null)
			return;
		tid = t;
		try
		{
			OutputStream outstrm = sock.getOutputStream();
			outstrm = new BufferedOutputStream(outstrm);
			strm = new DataOutputStream(outstrm);
		}
		catch (IOException ioe)
		{
			strm = null;
			tid = null;
			JPVMDebug.error("JPVMSendConnection, i/o exception");
		}
	}

	public static JPVMSendConnection connect(JPVMTaskId t, JPVMTaskId f)
			throws JPVMException
	{
		JPVMSendConnection ret = null;
		try
		{
			JPVMDebug.note("JPVMSendConnection, " + "connecting to "
					+ t.toString());

			// Make the new connection...
			Socket sock = new Socket(t.getHost(), t.getPort());
			ret = new JPVMSendConnection(sock, t);

			// Send my identity to the newly connected task...
			f.send(ret.strm);
			ret.strm.flush();
		}
		catch (IOException ioe)
		{
			JPVMDebug.note("JPVMSendConnection, connect - " + " i/o exception");
			throw new JPVMException("JPVMSendConnection, connect - "
					+ " i/o exception: \"" + ioe + "\"");
		}
		return ret;
	}
};
