/* JPVMTaskId.java
 *
 * Implements the unique jpvm task identifier class.
 *
 * Adam J Ferrari
 * Sat 05-25-1996
 *
 * Copyright (C) 1996  Adam J Ferrari
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 * MA 02139, USA.
 */

package jpvm;

import java.net.*;
import java.io.*;

public class JPVMTaskId implements Serializable
{
	private String	taskHost;
	private int		taskConnectPort;

	public JPVMTaskId()
	{
		taskHost = null;
		taskConnectPort = 0;
	};

	public JPVMTaskId(int my_port)
	{
		taskHost = null;
		taskConnectPort = 0;
		try
		{
			InetAddress taskAddr = InetAddress.getLocalHost();
			taskHost = taskAddr.getHostAddress(); // change here!
			taskConnectPort = my_port;
		}
		catch (UnknownHostException uhe)
		{
			JPVMDebug.error("JPVMTaskId, unknown host exception");
		}
	}

	public JPVMTaskId(String host, int port)
	{
		taskHost = new String(host);
		taskConnectPort = port;
	}

	public String getHost()
	{
		return taskHost;
	}

	public int getPort()
	{
		return taskConnectPort;
	}

	public String toString()
	{
		return ((taskHost != null ? taskHost : "(null)") + ", port #" + taskConnectPort);
	}

	public boolean equals(JPVMTaskId tid)
	{
		if (tid == null)
			return false;
		if (taskConnectPort != tid.taskConnectPort)
			return false;
		if (tid.taskHost == null)
			return false;
		boolean ret = tid.taskHost.equalsIgnoreCase(taskHost);
		return ret;
	}

	public void send(DataOutputStream strm) throws JPVMException
	{
		int i;
		try
		{
			int len = 0;
			if (taskHost != null)
			{
				len = taskHost.length();
				strm.writeInt(len);
				char hname[] = new char[len];
				taskHost.getChars(0, len, hname, 0);
				for (i = 0; i < len; i++)
				{
					strm.writeChar(hname[i]);
				}
			}
			else
			{
				strm.writeInt(len);
			}
			strm.writeInt(taskConnectPort);
		}
		catch (IOException ioe)
		{
			JPVMDebug.note("JPVMTaskId, send - i/o exception");
			throw new JPVMException("JPVMTaskId, send - i/o exception");
		}
	}

	public void recv(DataInputStream strm) throws JPVMException
	{
		int i;
		try
		{
			int len = strm.readInt();
			if (len > 0)
			{
				char hname[] = new char[len];
				for (i = 0; i < len; i++)
				{
					hname[i] = strm.readChar();
				}
				taskHost = new String(hname);
			}
			else
			{
				taskHost = null;
			}
			taskConnectPort = strm.readInt();
		}
		catch (IOException ioe)
		{
			JPVMDebug.note("JPVMTaskId, recv - i/o exception");
			throw new JPVMException("JPVMTaskId, recv - i/o exception");
		}
	}
};
